m = load_gmsh2('kondensatoren.msh', [1 2]);
P(1,:) = m.POS(:,1); P(2,:) = m.POS(:,2);
T      = m.TRIANGLES(:,1:3); GT     = m.TRIANGLES(:,4);
L      = m.LINES(:,1:2); GL     = m.LINES(:,3);
[ mesh ] = buildMesh( T,P,[],GT,L,GL );
clear m T P GT L GL
outside = 18; capacitor1 = 19; capacitor2 = 20;

while(true)
  A = assembDiffusion (1, mesh.T, mesh.P);
  x = mesh.P(1,:); y = mesh.P(2,:);
  b = assembRightSide(mesh,0);

  [ A, b ] = applyDirichletBoundaryConditions(A,b, mesh, outside,@(x,y)0);
  [ A, b ] = applyDirichletBoundaryConditions(A,b, mesh, capacitor1,@(x,y)50);
  [ A, b ] = applyDirichletBoundaryConditions(A,b, mesh, capacitor2,@(x,y)-50);

  u=A\b;
  [ ~, toRefine ]  = z2indicator( mesh, u, 10^-5, round(mesh.NuTri*0.025));
  if sum(toRefine)>round(mesh.NuTri*0.001)
    mesh = refineMesh( mesh, toRefine );
  else
    break
  end
end
[ graduTri, gradu ]  = gradientReconstruction( mesh, u );
figure(1)
trisurf(mesh.T,mesh.P(1,:),mesh.P(2,:),u, 'lineStyle','none');
colormap gray
xlabel('x [m]');ylabel('y [m]');zlabel('elektrisches Potential [V]');
figure(2)
triplot(mesh.T,mesh.P(1,:),mesh.P(2,:),'k');
colormap gray
xlabel('x [m]');ylabel('y [m]');xlim([0, 0.045]);
figure(3)
trisurf(mesh.T,mesh.P(1,:),mesh.P(2,:),sqrt(gradu(1,:).^2 + gradu(2,:).^2));
colormap gray
xlabel('x [m]');ylabel('y [m]');zlabel('elektrische Feldstaerke [V/m]');
figure(4)
colormap gray
patch('Faces',mesh.T,'Vertices',mesh.P','FaceVertexCData',sqrt(graduTri(1,:).^2 + graduTri(2,:).^2)','FaceColor','flat');
xlabel('x [m]');ylabel('y [m]');zlabel('elektrische Feldstaerke [V/m]');
figure(5)
myindex=find(abs(mesh.P(1,:)-0.0225)<0.00005);
plot(mesh.P(2,myindex),sqrt(gradu(1,myindex).^2 + gradu(2,myindex).^2),'k+')
xlabel('y [m]');ylabel('elektrische Feldstaerke [V/m]');