function [ mesh ] = refineMesh( mesh, toRefine )

index = find(toRefine); %*\label{refineMesh1}
markedEdges  = zeros(mesh.NuEdges,1);
markedEdgesI = unique(reshape(mesh.triangle2edge(index,:),[],1));
markedEdges(markedEdgesI) = 1;
node1 = mesh.edges(:,1); node2 = mesh.edges(:,2);
h = (mesh.P(1,node1)-mesh.P(1,node2) ).^2 + ( mesh.P(2,node1)-mesh.P(2,node2) ).^2;
h = sqrt(h);
keepOn = true;
while(keepOn) %*\label{refineMesh73}
  keepOn = false;
  for i=1:mesh.NuTri
    whichEdge = mesh.triangle2edge(i,:);
    howMany=sum( markedEdges(whichEdge)); %*\label{refineMesh2}
    if ((howMany == 2) || (howMany == 1))
      [~,biggest] = sort(h(whichEdge)); %*\label{refineMesh3}
      biggest = biggest(3);
      if ~markedEdges(whichEdge(biggest)) %*\label{refineMesh4}
        markedEdges(whichEdge(biggest))=1; %*\label{refineMesh5}
        keepOn = true;
      end
    end
  end
end%*\label{refineMesh74}
indexE = find(markedEdges); %*\label{refineMesh6}
node1 = mesh.edges(indexE,1); node2 = mesh.edges(indexE,2); %*\label{refineMesh7}
midpoint = (mesh.P(:,node1) + mesh.P(:,node2))/2;%*\label{refineMesh8}
mesh.P = horzcat(mesh.P,midpoint); %*\label{refineMesh9}
newMidpointPos = sparse(length(mesh.edges),1);%*\label{refineMesh10}
newMidpointPos(indexE) = [1:length(indexE)] + mesh.NuDoF;%*\label{refineMesh11}
[~,ia ,ib] = intersect(indexE,mesh.BoundaryEdges);
start  = length(mesh.Boundary)+1; finish = length(mesh.Boundary)+length(ia);
temp=zeros(size(mesh.Boundary)); %*\label{refineMesh12}
temp(mesh.Boundary)=mesh.GBoundary;
mesh.GBoundary(start:finish)= temp(ib);%*\label{refineMesh13}
mesh.Boundary(start:finish) = (ia+mesh.NuDoF)'; %*\label{refineMesh14}
oldEdges = mesh.edges;
start = mesh.NuDoF+1;
finish = (mesh.NuDoF+ size(midpoint,2));
mesh.edges(indexE,2)= [start:finish]'; %*\label{refineMesh15}
mesh.edges([1:length(indexE)] + mesh.NuEdges,1) = oldEdges(indexE,2);%*\label{refineMesh16}
mesh.edges([1:length(indexE)] + mesh.NuEdges,2) = start:finish;%*\label{refineMesh17}
newSegmentPos = sparse(length(mesh.edges),1);%*\label{refineMesh20}
newSegmentPos(indexE) = [1:length(indexE)] + mesh.NuEdges;%*\label{refineMesh21}
temp=zeros(size(oldEdges));%*\label{refineMesh18}
temp(mesh.BoundaryEdges)=mesh.GBoundaryEdges;
start=length(mesh.GBoundaryEdges)+1; finish=length(mesh.GBoundaryEdges)+length(ib);
mesh.GBoundaryEdges(start:finish)= temp(mesh.BoundaryEdges(ib));%*\label{refineMesh19}
for i=1:mesh.NuTri
  markedEdgesI = markedEdges(mesh.triangle2edge(i,:));
  howMany = sum(markedEdgesI);
  if howMany == 3
    nodeOldNotDiv = unique(reshape(oldEdges(mesh.triangle2edge(i,:),:),[],1));%*\label{refineMesh22}
    newSegmentIdx(1:3) = full(newSegmentPos(mesh.triangle2edge(i,1:3)));%*\label{refineMesh23}
    
    newEdge = zeros(3,1);
    for j=1:3 %*\label{refineMesh24}
      newTri(1) = nodeOldNotDiv(j);
      edgesOldPos  = find( nodeOldNotDiv(j)==mesh.edges(mesh.triangle2edge(i,:),1)); %*\label{refineMesh26}
      edgesNewPos  = find( nodeOldNotDiv(j)==mesh.edges(newSegmentIdx,1)); %*\label{refineMesh25}
      edgesOldPosNu = length(edgesOldPos);
      edgesNewPosNu = length(edgesNewPos);
      
      mesh.triangle2edge(end+1,1:3)=[0,0,0];
      if edgesOldPosNu>0 %*\label{refineMesh27}
        newTri(2:edgesOldPosNu+1) = mesh.edges(mesh.triangle2edge(i,edgesOldPos),2);%*\label{refineMesh29}
        mesh.triangle2edge(end,1:edgesOldPosNu) = mesh.triangle2edge(i,edgesOldPos);%*\label{refineMesh31}
      end
      if edgesNewPosNu>0 %*\label{refineMesh28}
        newTri((edgesOldPosNu+2):3) = mesh.edges(newSegmentIdx(edgesNewPos),2);%*\label{refineMesh30}
        mesh.triangle2edge(end,(edgesOldPosNu+1):2) = newSegmentIdx(edgesNewPos); %*\label{refineMesh32}
      end %*\label{refineMesh33}
      
      mesh.T(end+1,:) = newTri; %*\label{refineMesh34}
      newEdge(j) = length(mesh.edges)+1;
      mesh.edges(newEdge(j),1:2) = sort( [newTri(2), newTri(3)]); %*\label{refineMesh35}
      mesh.triangle2edge(end,3) = newEdge(j);
      mesh.GT(end+1)=mesh.GT(i);
    end
    mesh.triangle2edge(i,1:3) = newEdge(1:3);
    newTri = unique(reshape(mesh.edges(newEdge(1:3),:),[],1));
    mesh.T(i,:) = newTri;  
  elseif howMany == 2
    edgeLIdxNotDiv = find(~markedEdgesI);
    edgeGIdxNotDiv = mesh.triangle2edge(i,edgeLIdxNotDiv);
    edgeLIdxDiv = find(markedEdgesI);
    edgeGIdxDiv = mesh.triangle2edge(i,edgeLIdxDiv);
    edgesGIdxSplit = [edgeGIdxDiv, full(newSegmentPos(edgeGIdxDiv)')];%*\label{refineMesh47}
    
    [~,biggest] = sort(h(mesh.triangle2edge(i,:)));  %*\label{refineMesh36}
    biggest = biggest(3);
    if edgeLIdxDiv(1) ~= biggest
      edgeLIdxDiv(2) = edgeLIdxDiv(1); edgeLIdxDiv(1) = biggest;
    end;  %*\label{refineMesh37}
    nodeMl  = newMidpointPos(mesh.triangle2edge(i,edgeLIdxDiv(1)));%*\label{refineMesh38}
    nodeMs = newMidpointPos(mesh.triangle2edge(i,edgeLIdxDiv(2)));
    node1 = oldEdges(mesh.triangle2edge(i,edgeLIdxNotDiv),1);
    node2 = oldEdges(mesh.triangle2edge(i,edgeLIdxNotDiv),2);
    node3 = setdiff(mesh.T(i,:), [node1, node2]);%*\label{refineMesh39}
    nodesDiv2 = oldEdges(mesh.triangle2edge(i,edgeLIdxDiv(2)),:); %*\label{refineMesh40}
    if any(nodesDiv2 == node1) 
      temp = node1; node1=node2; node2=temp;
    end %*\label{refineMesh41}
    
    e1  = [node2, nodeMl];%*\label{refineMesh42}
    e2 = sort([nodeMs, nodeMl]);%*\label{refineMesh43}
    edgeGIdxNewE1  = length(mesh.edges)+1;%*\label{refineMesh44}
    mesh.edges(edgeGIdxNewE1,:)  = e1;
    edgeGIdxNewE2 = length(mesh.edges)+1;
    mesh.edges(edgeGIdxNewE2,:) = e2;%*\label{refineMesh45}
    mesh.T(i,:) = [nodeMl, node1, node2];%*\label{refineMesh46}
    [row,~] = find(mesh.edges(edgesGIdxSplit,:) == node1);%*\label{refineMesh48}
    edgeGIdxNode1 = edgesGIdxSplit(row);%*\label{refineMesh49}
    edgesGIdxSplit(row) = [];%*\label{refineMesh50}
    mesh.triangle2edge(i,:) = [edgeGIdxNewE1, edgeGIdxNotDiv, edgeGIdxNode1];%*\label{refineMesh51}
    
    mesh.T(end+1,:) = [nodeMl, nodeMs, node2];%*\label{refineMesh52}
    [row,~] = find(mesh.edges(edgesGIdxSplit,:) == node2);
    edgeGIdxNode2 = edgesGIdxSplit(row);
    edgesGIdxSplit(row) = [];
    mesh.triangle2edge(end+1,:) = [edgeGIdxNewE1, edgeGIdxNewE2, edgeGIdxNode2];%*\label{refineMesh53}
    mesh.GT(end+1) = mesh.GT(i);%*\label{refineMesh54}
    
    mesh.T(end+1,:) = [nodeMl, nodeMs, node3];%*\label{refineMesh55}
    mesh.triangle2edge(end+1,:)=[edgeGIdxNewE2,edgesGIdxSplit(1),edgesGIdxSplit(2)];
    mesh.GT(end+1) = mesh.GT(i); %*\label{refineMesh56}   
  elseif howMany == 1
    edgeLIdxDiv = find(markedEdgesI); %*\label{refineMesh57} 
    edgeGIdxDiv = mesh.triangle2edge(i,edgeLIdxDiv);
    edgesLIdxNotDiv = find(~markedEdgesI);
    edgesGIdxNotDiv = mesh.triangle2edge(i,edgesLIdxNotDiv);
    edgesGIdxSplit = [edgeGIdxDiv, full(newSegmentPos(edgeGIdxDiv))]; %*\label{refineMesh58} 
    
    if ~any(oldEdges(edgesGIdxNotDiv(1),:) == mesh.edges(edgesGIdxSplit(1),1)) %*\label{refineMesh59} 
      edgesGIdxSplit([1,2]) = edgesGIdxSplit([2,1]);
    end %*\label{refineMesh60}
    
    nodesOld = mesh.T(i,:); %*\label{refineMesh61} 
    nodesOldDiv = oldEdges(edgeGIdxDiv,:);
    nodeOldNotDiv = nodesOld(nodesOld~=nodesOldDiv(1) & nodesOld~=nodesOldDiv(2)); %*\label{refineMesh62} 
    nodeCenterDivEdge = newMidpointPos(mesh.triangle2edge(i,edgeLIdxDiv));
    edgeNew = [nodeOldNotDiv, nodeCenterDivEdge];
    edgeGIdxNew = length(mesh.edges)+1;
    mesh.edges(edgeGIdxNew,:) = edgeNew;
    mesh.T(end+1,:) = [nodeCenterDivEdge, oldEdges(edgesGIdxNotDiv(1),:)];%*\label{refineMesh63}
    mesh.triangle2edge(end+1,:) = [edgesGIdxNotDiv(1), edgesGIdxSplit(1), edgeGIdxNew];
    mesh.GT(end+1) = mesh.GT(i);%*\label{refineMesh64}
    
    mesh.T(i,:) = [nodeCenterDivEdge, oldEdges(edgesGIdxNotDiv(2),:)];%*\label{refineMesh65}
    mesh.triangle2edge(i,:) = [edgesGIdxNotDiv(2), edgesGIdxSplit(2), edgeGIdxNew];%*\label{refineMesh66}
  end
end
Point2Element(length(mesh.P)).Elements=[];%*\label{refineMesh67}
for i=1:length(mesh.P)
  Point2Element(i).Elements=[];
end
for i=1:length(mesh.T)
  for j=1:3
    index = length(Point2Element(mesh.T(i,j)).Elements)+1;
    Point2Element(mesh.T(i,j)).Elements(index)=i;
  end
end
mesh.Point2Element=Point2Element;%*\label{refineMesh68}
B(:,1,:)=mesh.P(:,mesh.T(:,2)) - mesh.P(:,mesh.T(:,1));%*\label{refineMesh69}
B(:,2,:)=mesh.P(:,mesh.T(:,3)) - mesh.P(:,mesh.T(:,1));
Bdet = B(1,1,:).*B(2,2,:) - B(1,2,:).*B(2,1,:);
mesh.B=B; %*\label{refineMesh70}
mesh.Bdet=squeeze(Bdet);
mesh.NuDoF    = length(mesh.P);
mesh.NuBNodes = length(mesh.Boundary);
mesh.NuEdges  = length(mesh.edges);
mesh.NuTri    = length(mesh.T);
mesh.edge2triangle=-1*ones(mesh.NuEdges,2); %*\label{refineMesh71}
for i=1:mesh.NuTri
  index = mesh.triangle2edge(i,:);
  for j=1:3
    if mesh.edge2triangle(index(j),1) == -1
      mesh.edge2triangle(index(j),1) = i;
    else
      mesh.edge2triangle(index(j),2) = i;
    end
  end
end
mesh.BoundaryEdges = find(mesh.edge2triangle(:,2) == -1);%*\label{refineMesh72}
mesh.neighbours = zeros(mesh.NuTri,3);
for i = 1:mesh.NuTri
  tries = mesh.edge2triangle(mesh.triangle2edge(i,:),:); 
  tries(tries == i) = [];
  mesh.neighbours(i,:) = sort(tries, 2, 'descend');
end
