m = load_gmsh2('relais.msh', [1 2]);
P(1,:) = m.POS(:,1);         P(2,:) = m.POS(:,2);
T      = m.TRIANGLES(:,1:3); GT     = m.TRIANGLES(:,4);
L      = m.LINES(:,1:2);     GL     = m.LINES(:,3);

[ mesh ] = buildMesh( T,P,[],GT,L,GL );
clear m T P GT L GL

boundaryGroup = 54;
armatureGroup = 52;   % Anker
yokeGroup = 62;       % Joch
rechteCoilGroup = 66; % rechte Spule
leftCoilGroup   = 53; % linke Spule

mu = [
  0.087	8.7000E-04; 0.332	1.6600E-03; 0.725	2.4167E-03; 0.985	2.4625E-03;
  1.144	2.2880E-03; 1.241	2.0683E-03; 1.369	1.7113E-03; 1.429	1.4290E-03;
  1.471	1.2258E-03; 1.504	1.0743E-03; 1.524	9.5250E-04; 1.542	8.5667E-04;
  1.557	7.7850E-04; 1.572	7.1455E-04; 1.592	6.1231E-04; 1.61	5.3667E-04;
  1.634	4.6686E-04; 1.65	4.1250E-04; 1.683	3.3660E-04; 1.731	2.4729E-04;
  1.776	1.9733E-04; 1.809	1.6445E-04; 1.872	1.2480E-04; 1.935	9.6750E-05;
  2.022	6.7400E-05; 2.073	5.1825E-05; 2.1		4.2000E-05; 2.116	3.6483E-05;
  4 4*pi*10^-7];
muAir = 4*pi*10^-7;
vareps = (1/muAir)*ones(length(mesh.T),1);
vareps(mesh.GT == armatureGroup) = 1/1E-04; % Anker
vareps(mesh.GT == yokeGroup) = 1/1E-04; % Joch

x = mesh.P(1,:); y = mesh.P(2,:);
isPointInRightInductor = false(size(x));
isPointInRightInductor(mesh.T(mesh.GT == rechteCoilGroup)) = true;
isPointInLeftInductor = false(size(x));
isPointInLeftInductor(mesh.T(mesh.GT == leftCoilGroup)) = true;
f = zeros(size(x));
jz = 2*10^7; 
f(isPointInRightInductor) = -jz; % Spule2
f(isPointInLeftInductor)  = jz; % Spule1

ualt = zeros(size(x))'; 
i=1; maxItr = 50; uDiff=1;
diffTol = 10^-6;

while (uDiff>diffTol) && (i<maxItr)
  [A, b] = assembPDE( vareps, [], [], f, mesh);
  [A, b] = applyDirichletBoundaryConditions(A,b,mesh,boundaryGroup,zeros(size(x)));
  u = A\b;
  
  [ gradu, ~ ]  = gradientPerElement( mesh, u ); %*\label{test93:mukonstr:start}
  BNorm = ( gradu(1,:).^2 + gradu(2,:).^2 ).^(0.5);
  vareps = 1./interp1(mu(:,1),mu(:,2),BNorm,'linear'); %*\label{test93:mukonstr:interpol}
  vareps(BNorm<mu(1,1))  = 1/mu(1,2); %*\label{test93:mukonstr:luft1}
  vareps(BNorm>mu(end,1))= 1/mu(end,2); %*\label{test93:mukonstr:luft2}
  notChaning =  ~( (mesh.GT == armatureGroup) | (mesh.GT == yokeGroup) );
  vareps(notChaning) = (1/muAir); %*\label{test93:mukonstr:ende}
  
  uDiff = max(abs(ualt-u));
  ualt = u; i=i+1;
end

figure(1);
trisurf(mesh.T,x,y,u,'EdgeColor','none');
view(2);  colorbar;
xlim([-0.0114, 0.0114]); ylim([-0.015 0.0066]); axis equal;
xlabel('x [m]'); ylabel('y [m]'); title('A in [V s/m]');
figure(2);
[ gradu, ~ ]  = gradientPerElement( mesh, u );
BNorm = ( gradu(1,:).^2 + gradu(2,:).^2 ).^(0.5);
patch('Faces',mesh.T,'Vertices',mesh.P','FaceVertexCData',BNorm','FaceColor','flat','EdgeColor','none');
xlabel('x [m]'); ylabel('y [m]'); title('||B||_2 in Tesla');
colorbar; xlim([-0.0114, 0.0114]);ylim([-0.015 0.0066]);axis equal;
figure(3);
patch('Faces',mesh.T,'Vertices',mesh.P','FaceVertexCData',(1./vareps')/muAir,'FaceColor','flat','EdgeColor','none');
xlabel('x [m]'); ylabel('y [m]'); title('\mu_r(||B||_2)');
colorbar; xlim([-0.0114, 0.0114]);ylim([-0.015 0.0066]);axis equal;